@@ -8,6 +8,14 @@
 #ifndef LIBCOMPAT_STDLIB_H
 #define LIBCOMPAT_STDLIB_H
 
+#include <stdint.h>
+
+#ifndef HAVE_ARC4RANDOM_
+uint32_t arc4random(void);
+uint32_t arc4random_uniform(uint32_t);
+void arc4random_buf(void *buf, size_t nbytes);
+#endif
+
 #ifndef HAVE_REALLOCARRAY
 void *reallocarray(void *, size_t, size_t);
 #endif
@@ -17,17 +25,16 @@ void *recallocarray(void *, size_t, size_t, size_t);
 #endif
 
 #ifndef HAVE_FREEZERO
-void freezero(void *ptr, size_t sz);
+void freezero(void *, size_t);
 #endif
 
 #ifndef HAVE_SETPROCTITLE
-void compat_init_setproctitle(int argc, char *argv[]);
-void setproctitle(const char *fmt, ...);
+void compat_init_setproctitle(int, char *[]);
+void setproctitle(const char *, ...);
 #endif
 
 #ifndef HAVE_STRTONUM
-long long strtonum(const char *nptr, long long minval,
-		long long maxval, const char **errstr);
+long long strtonum(const char *, long long, long long, const char **);
 #endif
 
 #endif
