@@ -1,4 +1,4 @@
-/*	$OpenBSD: json.h,v 1.1 2023/04/27 07:57:25 claudio Exp $ */
+/*	$OpenBSD: json.h,v 1.3 2023/06/05 14:19:13 claudio Exp $ */
 
 /*
  * Copyright (c) 2020 Claudio Jeker <claudio@openbsd.org>
@@ -22,10 +22,11 @@
 void	json_do_start(FILE *);
 int	json_do_finish(void);
 void	json_do_array(const char *);
-void	json_do_object(const char *);
+void	json_do_object(const char *, int);
 void	json_do_end(void);
 void	json_do_printf(const char *, const char *, ...)
 	    __attribute__((__format__ (printf, 2, 3)));
+void	json_do_string(const char *, const char *);
 void	json_do_hexdump(const char *, void *, size_t);
 void	json_do_bool(const char *, int);
 void	json_do_uint(const char *, unsigned long long);
