@@ -1,4 +1,4 @@
-/*	$OpenBSD: mft.c,v 1.106 2024/02/05 19:23:58 job Exp $ */
+/*	$OpenBSD: mft.c,v 1.112 2024/02/22 12:49:42 job Exp $ */
 /*
  * Copyright (c) 2022 Theo Buehler <tb@openbsd.org>
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
@@ -34,21 +34,15 @@
 
 #include "extern.h"
 
-/*
- * Parse results and data of the manifest file.
- */
-struct	parse {
-	const char	*fn; /* manifest file name */
-	struct mft	*res; /* result object */
-	int		 found_crl;
-};
-
 extern ASN1_OBJECT	*mft_oid;
 
 /*
  * Types and templates for the Manifest eContent, RFC 6486, section 4.2.
  */
 
+ASN1_ITEM_EXP FileAndHash_it;
+ASN1_ITEM_EXP Manifest_it;
+
 typedef struct {
 	ASN1_IA5STRING	*file;
 	ASN1_BIT_STRING	*hash;
@@ -57,8 +51,13 @@ typedef struct {
 DECLARE_STACK_OF(FileAndHash);
 
 #ifndef DEFINE_STACK_OF
+#define sk_FileAndHash_dup(sk)		SKM_sk_dup(FileAndHash, (sk))
+#define sk_FileAndHash_free(sk)		SKM_sk_free(FileAndHash, (sk))
 #define sk_FileAndHash_num(sk)		SKM_sk_num(FileAndHash, (sk))
 #define sk_FileAndHash_value(sk, i)	SKM_sk_value(FileAndHash, (sk), (i))
+#define sk_FileAndHash_sort(sk)		SKM_sk_sort(FileAndHash, (sk))
+#define sk_FileAndHash_set_cmp_func(sk, cmp) \
+    SKM_sk_set_cmp_func(FileAndHash, (sk), (cmp))
 #endif
 
 typedef struct {
@@ -122,6 +121,8 @@ rtype_from_file_extension(const char *fn)
 		return RTYPE_TAK;
 	if (strcasecmp(fn + sz - 4, ".csv") == 0)
 		return RTYPE_GEOFEED;
+	if (strcasecmp(fn + sz - 4, ".spl") == 0)
+		return RTYPE_SPL;
 
 	return RTYPE_INVALID;
 }
@@ -163,6 +164,7 @@ rtype_from_mftfile(const char *fn)
 	case RTYPE_GBR:
 	case RTYPE_ROA:
 	case RTYPE_ASPA:
+	case RTYPE_SPL:
 	case RTYPE_TAK:
 		return type;
 	default:
@@ -175,147 +177,222 @@ rtype_from_mftfile(const char *fn)
  * Return zero on failure, non-zero on success.
  */
 static int
-mft_parse_filehash(struct parse *p, const FileAndHash *fh)
+mft_parse_filehash(const char *fn, struct mft *mft, const FileAndHash *fh,
+    int *found_crl)
 {
-	char			*fn = NULL;
+	char			*file = NULL;
 	int			 rc = 0;
 	struct mftfile		*fent;
 	enum rtype		 type;
 	size_t			 new_idx = 0;
 
 	if (!valid_mft_filename(fh->file->data, fh->file->length)) {
-		warnx("%s: RFC 6486 section 4.2.2: bad filename", p->fn);
+		warnx("%s: RFC 6486 section 4.2.2: bad filename", fn);
 		goto out;
 	}
-	fn = strndup(fh->file->data, fh->file->length);
-	if (fn == NULL)
+	file = strndup(fh->file->data, fh->file->length);
+	if (file == NULL)
 		err(1, NULL);
 
 	if (fh->hash->length != SHA256_DIGEST_LENGTH) {
 		warnx("%s: RFC 6486 section 4.2.1: hash: "
-		    "invalid SHA256 length, have %d",
-		    p->fn, fh->hash->length);
+		    "invalid SHA256 length, have %d", fn, fh->hash->length);
 		goto out;
 	}
 
-	type = rtype_from_mftfile(fn);
+	type = rtype_from_mftfile(file);
 	/* remember the filehash for the CRL in struct mft */
-	if (type == RTYPE_CRL && strcmp(fn, p->res->crl) == 0) {
-		memcpy(p->res->crlhash, fh->hash->data, SHA256_DIGEST_LENGTH);
-		p->found_crl = 1;
+	if (type == RTYPE_CRL && strcmp(file, mft->crl) == 0) {
+		memcpy(mft->crlhash, fh->hash->data, SHA256_DIGEST_LENGTH);
+		*found_crl = 1;
 	}
 
 	if (filemode)
-		fent = &p->res->files[p->res->filesz++];
+		fent = &mft->files[mft->filesz++];
 	else {
 		/* Fisher-Yates shuffle */
-		new_idx = arc4random_uniform(p->res->filesz + 1);
-		p->res->files[p->res->filesz++] = p->res->files[new_idx];
-		fent = &p->res->files[new_idx];
+		new_idx = arc4random_uniform(mft->filesz + 1);
+		mft->files[mft->filesz++] = mft->files[new_idx];
+		fent = &mft->files[new_idx];
 	}
 
 	fent->type = type;
-	fent->file = fn;
-	fn = NULL;
+	fent->file = file;
+	file = NULL;
 	memcpy(fent->hash, fh->hash->data, SHA256_DIGEST_LENGTH);
 
 	rc = 1;
  out:
-	free(fn);
+	free(file);
 	return rc;
 }
 
+static int
+mft_fh_cmp_name(const FileAndHash *const *a, const FileAndHash *const *b)
+{
+	if ((*a)->file->length < (*b)->file->length)
+		return -1;
+	if ((*a)->file->length > (*b)->file->length)
+		return 1;
+
+	return memcmp((*a)->file->data, (*b)->file->data, (*b)->file->length);
+}
+
+static int
+mft_fh_cmp_hash(const FileAndHash *const *a, const FileAndHash *const *b)
+{
+	assert((*a)->hash->length == SHA256_DIGEST_LENGTH);
+	assert((*b)->hash->length == SHA256_DIGEST_LENGTH);
+
+	return memcmp((*a)->hash->data, (*b)->hash->data, (*b)->hash->length);
+}
+
+/*
+ * Assuming that the hash lengths are validated, this checks that all file names
+ * and hashes in a manifest are unique. Returns 1 on success, 0 on failure.
+ */
+static int
+mft_has_unique_names_and_hashes(const char *fn, const Manifest *mft)
+{
+	STACK_OF(FileAndHash)	*fhs;
+	int			 i, ret = 0;
+
+	if ((fhs = sk_FileAndHash_dup(mft->fileList)) == NULL)
+		err(1, NULL);
+
+	(void)sk_FileAndHash_set_cmp_func(fhs, mft_fh_cmp_name);
+	sk_FileAndHash_sort(fhs);
+
+	for (i = 0; i < sk_FileAndHash_num(fhs) - 1; i++) {
+		const FileAndHash *curr = sk_FileAndHash_value(fhs, i);
+		const FileAndHash *next = sk_FileAndHash_value(fhs, i + 1);
+
+		if (mft_fh_cmp_name(&curr, &next) == 0) {
+			warnx("%s: duplicate name: %.*s", fn,
+			    curr->file->length, curr->file->data);
+			goto err;
+		}
+	}
+
+	(void)sk_FileAndHash_set_cmp_func(fhs, mft_fh_cmp_hash);
+	sk_FileAndHash_sort(fhs);
+
+	for (i = 0; i < sk_FileAndHash_num(fhs) - 1; i++) {
+		const FileAndHash *curr = sk_FileAndHash_value(fhs, i);
+		const FileAndHash *next = sk_FileAndHash_value(fhs, i + 1);
+
+		if (mft_fh_cmp_hash(&curr, &next) == 0) {
+			warnx("%s: duplicate hash for %.*s and %.*s", fn,
+			    curr->file->length, curr->file->data,
+			    next->file->length, next->file->data);
+			goto err;
+		}
+	}
+
+	ret = 1;
+
+ err:
+	sk_FileAndHash_free(fhs);
+
+	return ret;
+}
+
 /*
  * Handle the eContent of the manifest object, RFC 6486 sec. 4.2.
  * Returns 0 on failure and 1 on success.
  */
 static int
-mft_parse_econtent(const unsigned char *d, size_t dsz, struct parse *p)
+mft_parse_econtent(const char *fn, struct mft *mft, const unsigned char *d,
+    size_t dsz)
 {
 	const unsigned char	*oder;
-	Manifest		*mft;
+	Manifest		*mft_asn1;
 	FileAndHash		*fh;
-	int			 i, rc = 0;
+	int			 found_crl, i, rc = 0;
 
 	oder = d;
-	if ((mft = d2i_Manifest(NULL, &d, dsz)) == NULL) {
-		warnx("%s: RFC 6486 section 4: failed to parse Manifest",
-		    p->fn);
+	if ((mft_asn1 = d2i_Manifest(NULL, &d, dsz)) == NULL) {
+		warnx("%s: RFC 6486 section 4: failed to parse Manifest", fn);
 		goto out;
 	}
 	if (d != oder + dsz) {
-		warnx("%s: %td bytes trailing garbage in eContent", p->fn,
+		warnx("%s: %td bytes trailing garbage in eContent", fn,
 		    oder + dsz - d);
 		goto out;
 	}
 
-	if (!valid_econtent_version(p->fn, mft->version, 0))
+	if (!valid_econtent_version(fn, mft_asn1->version, 0))
 		goto out;
 
-	p->res->seqnum = x509_convert_seqnum(p->fn, mft->manifestNumber);
-	if (p->res->seqnum == NULL)
+	mft->seqnum = x509_convert_seqnum(fn, mft_asn1->manifestNumber);
+	if (mft->seqnum == NULL)
 		goto out;
 
 	/*
 	 * OpenSSL's DER decoder implementation will accept a GeneralizedTime
 	 * which doesn't conform to RFC 5280. So, double check.
 	 */
-	if (ASN1_STRING_length(mft->thisUpdate) != GENTIME_LENGTH) {
-		warnx("%s: embedded from time format invalid", p->fn);
+	if (ASN1_STRING_length(mft_asn1->thisUpdate) != GENTIME_LENGTH) {
+		warnx("%s: embedded from time format invalid", fn);
 		goto out;
 	}
-	if (ASN1_STRING_length(mft->nextUpdate) != GENTIME_LENGTH) {
-		warnx("%s: embedded until time format invalid", p->fn);
+	if (ASN1_STRING_length(mft_asn1->nextUpdate) != GENTIME_LENGTH) {
+		warnx("%s: embedded until time format invalid", fn);
 		goto out;
 	}
 
-	if (!x509_get_time(mft->thisUpdate, &p->res->thisupdate)) {
-		warn("%s: parsing manifest thisUpdate failed", p->fn);
+	if (!x509_get_time(mft_asn1->thisUpdate, &mft->thisupdate)) {
+		warn("%s: parsing manifest thisUpdate failed", fn);
 		goto out;
 	}
-	if (!x509_get_time(mft->nextUpdate, &p->res->nextupdate)) {
-		warn("%s: parsing manifest nextUpdate failed", p->fn);
+	if (!x509_get_time(mft_asn1->nextUpdate, &mft->nextupdate)) {
+		warn("%s: parsing manifest nextUpdate failed", fn);
 		goto out;
 	}
 
-	if (p->res->thisupdate > p->res->nextupdate) {
-		warnx("%s: bad update interval", p->fn);
+	if (mft->thisupdate > mft->nextupdate) {
+		warnx("%s: bad update interval", fn);
 		goto out;
 	}
 
-	if (OBJ_obj2nid(mft->fileHashAlg) != NID_sha256) {
+	if (OBJ_obj2nid(mft_asn1->fileHashAlg) != NID_sha256) {
 		warnx("%s: RFC 6486 section 4.2.1: fileHashAlg: "
-		    "want SHA256 object, have %s (NID %d)", p->fn,
-		    ASN1_tag2str(OBJ_obj2nid(mft->fileHashAlg)),
-		    OBJ_obj2nid(mft->fileHashAlg));
+		    "want SHA256 object, have %s (NID %d)", fn,
+		    ASN1_tag2str(OBJ_obj2nid(mft_asn1->fileHashAlg)),
+		    OBJ_obj2nid(mft_asn1->fileHashAlg));
 		goto out;
 	}
 
-	if (sk_FileAndHash_num(mft->fileList) >= MAX_MANIFEST_ENTRIES) {
-		warnx("%s: %d exceeds manifest entry limit (%d)", p->fn,
-		    sk_FileAndHash_num(mft->fileList), MAX_MANIFEST_ENTRIES);
+	if (sk_FileAndHash_num(mft_asn1->fileList) >= MAX_MANIFEST_ENTRIES) {
+		warnx("%s: %d exceeds manifest entry limit (%d)", fn,
+		    sk_FileAndHash_num(mft_asn1->fileList),
+		    MAX_MANIFEST_ENTRIES);
 		goto out;
 	}
 
-	p->res->files = calloc(sk_FileAndHash_num(mft->fileList),
+	mft->files = calloc(sk_FileAndHash_num(mft_asn1->fileList),
 	    sizeof(struct mftfile));
-	if (p->res->files == NULL)
+	if (mft->files == NULL)
 		err(1, NULL);
 
-	for (i = 0; i < sk_FileAndHash_num(mft->fileList); i++) {
-		fh = sk_FileAndHash_value(mft->fileList, i);
-		if (!mft_parse_filehash(p, fh))
+	found_crl = 0;
+	for (i = 0; i < sk_FileAndHash_num(mft_asn1->fileList); i++) {
+		fh = sk_FileAndHash_value(mft_asn1->fileList, i);
+		if (!mft_parse_filehash(fn, mft, fh, &found_crl))
 			goto out;
 	}
 
-	if (!p->found_crl) {
-		warnx("%s: CRL not part of MFT fileList", p->fn);
+	if (!found_crl) {
+		warnx("%s: CRL not part of MFT fileList", fn);
 		goto out;
 	}
 
+	if (!mft_has_unique_names_and_hashes(fn, mft_asn1))
+		goto out;
+
 	rc = 1;
  out:
-	Manifest_free(mft);
+	Manifest_free(mft_asn1);
 	return rc;
 }
 
@@ -327,7 +404,7 @@ struct mft *
 mft_parse(X509 **x509, const char *fn, int talid, const unsigned char *der,
     size_t len)
 {
-	struct parse	 p;
+	struct mft	*mft;
 	struct cert	*cert = NULL;
 	int		 rc = 0;
 	size_t		 cmsz;
@@ -335,28 +412,25 @@ mft_parse(X509 **x509, const char *fn, int talid, const unsigned char *der,
 	char		*crldp = NULL, *crlfile;
 	time_t		 signtime = 0;
 
-	memset(&p, 0, sizeof(struct parse));
-	p.fn = fn;
-
 	cms = cms_parse_validate(x509, fn, der, len, mft_oid, &cmsz, &signtime);
 	if (cms == NULL)
 		return NULL;
 	assert(*x509 != NULL);
 
-	if ((p.res = calloc(1, sizeof(struct mft))) == NULL)
+	if ((mft = calloc(1, sizeof(*mft))) == NULL)
 		err(1, NULL);
-	p.res->signtime = signtime;
+	mft->signtime = signtime;
 
-	if (!x509_get_aia(*x509, fn, &p.res->aia))
+	if (!x509_get_aia(*x509, fn, &mft->aia))
 		goto out;
-	if (!x509_get_aki(*x509, fn, &p.res->aki))
+	if (!x509_get_aki(*x509, fn, &mft->aki))
 		goto out;
-	if (!x509_get_sia(*x509, fn, &p.res->sia))
+	if (!x509_get_sia(*x509, fn, &mft->sia))
 		goto out;
-	if (!x509_get_ski(*x509, fn, &p.res->ski))
+	if (!x509_get_ski(*x509, fn, &mft->ski))
 		goto out;
-	if (p.res->aia == NULL || p.res->aki == NULL || p.res->sia == NULL ||
-	    p.res->ski == NULL) {
+	if (mft->aia == NULL || mft->aki == NULL || mft->sia == NULL ||
+	    mft->ski == NULL) {
 		warnx("%s: RFC 6487 section 4.8: "
 		    "missing AIA, AKI, SIA, or SKI X509 extension", fn);
 		goto out;
@@ -388,16 +462,16 @@ mft_parse(X509 **x509, const char *fn, int talid, const unsigned char *der,
 		    "bad CRL distribution point extension", fn);
 		goto out;
 	}
-	if ((p.res->crl = strdup(crlfile)) == NULL)
+	if ((mft->crl = strdup(crlfile)) == NULL)
 		err(1, NULL);
 
-	if (mft_parse_econtent(cms, cmsz, &p) == 0)
+	if (mft_parse_econtent(fn, mft, cms, cmsz) == 0)
 		goto out;
 
 	if ((cert = cert_parse_ee_cert(fn, talid, *x509)) == NULL)
 		goto out;
 
-	if (p.res->signtime > p.res->nextupdate) {
+	if (mft->signtime > mft->nextupdate) {
 		warnx("%s: dating issue: CMS signing-time after MFT nextUpdate",
 		    fn);
 		goto out;
@@ -406,15 +480,15 @@ mft_parse(X509 **x509, const char *fn, int talid, const unsigned char *der,
 	rc = 1;
 out:
 	if (rc == 0) {
-		mft_free(p.res);
-		p.res = NULL;
+		mft_free(mft);
+		mft = NULL;
 		X509_free(*x509);
 		*x509 = NULL;
 	}
 	free(crldp);
 	cert_free(cert);
 	free(cms);
-	return p.res;
+	return mft;
 }
 
 /*
